<?php
App::uses('CustomEmail', 'Lib');

class InjectContactFormComponent extends Component {

	protected $Controller;
	const FORM_SUCCESS = 1;
	const FORM_ERROR = 2;
	const FORM_VALIDATION = 3;

	public function initialize(Controller $Controller){
		$this->Controller = $Controller;
	}

	protected function _showFlash($status) {
		if ($status == self::FORM_SUCCESS) {
			$this->Controller->Session->setFlash('Your message has been sent', 'flash_success');
		}
		else if ($status == self::FORM_ERROR) {
			$this->Controller->Session->setFlash('There was an issue saving your message, please try again', 'flash_fail');
		}
		else if ($status == self::FORM_VALIDATION) {
			$this->Controller->Session->setFlash('Please check you have filled out all the required fields', 'flash_fail');
		}
	}


	protected function _handleValidation($fields, $data) {
		// setup the dynamic validation
		foreach ($fields as $field => $options) {

			if (! empty($options['validate'])) {

				foreach ($options['validate'] as $rule => $rule_data) {

					$this->Controller->ContactFormResponse->validator()->add($field, $rule, $rule_data);
				}
			}
		}
		// validate!
		return $this->Controller->ContactFormResponse->validator()->validateAssociated($data);
	}

	protected function _removeValidation($fields) {
		foreach ($fields as $field => $options) {

			if (! empty($options['validate'])) {

				$this->Controller->ContactFormResponse->validator()->remove($field);
			}
		}
	}

	protected function  _saveResponse($form, $contact_form_data, $responseName) {
		
		$data = array(
			'ContactFormResponse' => array(
				'contact_form_id' => $form['ContactForm']['id'],
				'name' => $responseName,
				'form_data' => $contact_form_data,
				'created' => date('Y-m-d H:i:s')
			)
		);
		return $this->Controller->ContactFormResponse->save($data);

	}


	protected function _setEmailFrom() {
		 return array(
			'name' => Configure::read('SiteSetting.site_title'),
			'email' => Configure::read('SiteSetting.admin_email'),
		);
	}

	protected function _sendNotificationEmail($email_from, $email_to, $form, $form_data, $fields) {

		$email = new CustomEmail();
		$email->template('EvContactForm.contact_form_notification')
			->from($email_from['email'], (isset($email_from['name'])) ? $email_from['name'] : null)
			->emailFormat('html')
			->subject($form['ContactForm']['subject'])
			->viewVars(array(
				'form_data' => $form_data,
				'fields' => $fields
			));
		// build the to / cc's
		$to = array_shift($email_to);

		$email->to($to);

		if (! empty($email_to)) {
			foreach ($email_to as $recipient) {
				$email->addCc($recipient);
			}
		}
		$email->send();
	}

	protected function _loadForm($id) {
		
		$this->Controller->loadModel('ContactForm');
		$form = $this->Controller->ContactForm->find('first', array('conditions' => array(
				'ContactForm.id' => $id
		)));
		if (empty($form['ContactForm'])) {
			return false;
		}
		return $form;
	}

	protected function _loadConfig($form_name) {
		$config= Configure::read('EvContactForm');
		return $config[$form_name];
	}

	

	public function display($id, $name) {

		// Load configs, overridable!
		$form = $this->_loadForm($id);
		$formData = $this->_loadConfig($form['ContactForm']['form_data']);

		$fields = $formData['fields'];

		if ($this->Controller->request->is('post') && ! empty($this->Controller->request->data[$name])) {

			$this->Controller->loadModel('ContactFormResponse');
			
			if ($this->_handleValidation($fields, $this->Controller->request->data[$name])) {

				// passed validation, remove rules so we can add to the database
				$this->_removeValidation($fields);

				// serialize it
				$contact_form_data = serialize($this->Controller->request->data[$name]);

				// name label
				$name_lbl = $formData['name_label'];


				if (! empty($this->Controller->request->data[$name][$name_lbl])) {
					$responseName = $this->Controller->request->data[$name][$name_lbl];
				} else {
					$responseName = 'Unknown Name';
				}

				$this->Controller->set($name . '_completed', true);
				// save it
				if ($this->_saveResponse($form, $contact_form_data, $responseName)) {

					$email_from = $this->_setEmailFrom();

					$email_to = explode(',', $form['ContactForm']['email_to']);

					if (! is_null($email_from) && ! is_null($email_to)) {

						// send the email!
						$this->_sendNotificationEmail($email_from, $email_to, $form, $this->Controller->request->data[$name], $fields);
					}

					// unset form data nad show success
					unset($this->Controller->request->data[$name]);
					if (isset($formData['flash_message']) && $formData['flash_message']) {
						$this->_showFlash(self::FORM_SUCCESS);
					}
					$this->Controller->set($name . '_sent', true);
				} else {
					if (isset($formData['flash_message']) && $formData['flash_message']) {
						$this->_showFlash(self::FORM_ERROR);
					}
					// failed to save, error
					$contact_form_errors = array(
						'There was an issue saving your message, please try again'
					);

					$this->Controller->set($name . '_errors', $contact_form_errors);
				}

			} else {

				// compile the validate errors
				$this->_showFlash(self::FORM_VALIDATION);

				$contact_form_errors = array();

				foreach ($this->Controller->ContactFormResponse->validationErrors as $field => $errors) {
					foreach ($errors as $msg) {
						$contact_form_errors[] = $msg;
					}
				}

				// I dont know why I need this
				if (isset($this->Controller->request->data[$name][$name])) {
					$temp = $this->Controller->request->data[$name][$name];
					$this->Controller->request->data[$name] = $temp;
				}
				$this->Controller->set($name . '_errors', $contact_form_errors);
			}
		}

		$this->Controller->set($name . '_form', $formData);
	}
}
?>